@@ -11,6 +11,9 @@
 .MAX_INSPECT_COL_VIEW = 10000
 .LOGICAL_OPERATORS = c("==", ">", "<", "!=", ">=", "<=", "&", "|", "&&", "||", "!", "is.na")
 
+"%p0%"   <- function(x,y) assign(deparse(substitute(x)), paste(x, y, sep = ""), parent.frame())  # paste0
+"%p%"    <- function(x,y) assign(deparse(substitute(x)), paste(x, y), parent.frame()) # paste
+
 # Initialize functions for R logging
 .myPath = paste(Sys.getenv("HOME"), "Library", "Application Support", "h2o", sep=.Platform$file.sep)
 if(.Platform$OS.type == "windows")
@@ -106,6 +109,7 @@ h2o.setLogPath <- function(path, type) {
 .h2o.__DOMAIN_MAPPING = "2/DomainMapping.json"
 .h2o.__SET_DOMAIN = "2/SetDomains.json"
 .h2o.__PAGE_ALLMODELS = "2/Models.json"
+.h2o.__GAINS <- "2/GainsLiftTable.json"
 
 .h2o.__PAGE_IMPUTE= "2/Impute.json"
 .h2o.__PAGE_EXEC2 = "2/Exec2.json"
@@ -129,6 +133,9 @@ h2o.setLogPath <- function(path, type) {
 .h2o.__PAGE_INSPECTOR = "2/Inspector.json"
 .h2o.__PAGE_ANOMALY = "2/Anomaly.json"
 .h2o.__PAGE_DEEPFEATURES = "2/DeepFeatures.json"
+.h2o.__PAGE_SETTIMEZONE = "2/SetTimezone.json"
+.h2o.__PAGE_GETTIMEZONE = "2/GetTimezone.json"
+.h2o.__PAGE_LISTTIMEZONES = "2/ListTimezones.json"
 
 .h2o.__PAGE_CoxPH = "2/CoxPH.json"
 .h2o.__PAGE_CoxPHProgress = "2/CoxPHProgressPage.json"
@@ -162,12 +169,17 @@ h2o.setLogPath <- function(path, type) {
 .h2o.__PAGE_NBProgress = "2/NBProgressPage.json"
 .h2o.__PAGE_NBModelView = "2/NBModelView.json"
 .h2o.__PAGE_CreateFrame = "2/CreateFrame.json"
+.h2o.__PAGE_Interaction = "2/Interaction.json"
 .h2o.__PAGE_ReBalance = "2/ReBalance.json"
 .h2o.__PAGE_SplitFrame = "2/FrameSplitPage.json"
 .h2o.__PAGE_NFoldExtractor = "2/NFoldFrameExtractPage.json"
 .h2o.__PAGE_MissingVals = "2/InsertMissingValues.json"
 .h2o.__PAGE_SaveModel = "2/SaveModel.json"
 .h2o.__PAGE_LoadModel = "2/LoadModel.json"
+.h2o.__PAGE_RemoveVec = "2/RemoveVec.json"
+.h2o.__PAGE_Order = "2/Order.json"
+
+.h2o.__GLMMakeModel = "2/GLMMakeModel.json"
 
 # client -- Connection object returned from h2o.init().
 # page   -- URL to access within the H2O server.
@@ -204,7 +216,7 @@ h2o.setLogPath <- function(path, type) {
     
     hg = basicHeaderGatherer()
     tg = basicTextGatherer()
-    postForm(myURL, style = "POST", .opts = curlOptions(headerfunction = hg$update, writefunc = tg[[1]]), ...)
+    postForm(myURL, style = "POST", .opts = curlOptions(headerfunction = hg$update, writefunc = tg[[1]], useragent=R.version.string), ...)
     temp = tg$value()
     
     # Log HTTP response from H2O
@@ -216,7 +228,7 @@ h2o.setLogPath <- function(path, type) {
     if(!file.exists(cmdDir)) stop(cmdDir, " directory does not exist")
     write(s, file = .pkg.env$h2o.__LOG_COMMAND, append = TRUE)
   } else
-    temp = postForm(myURL, style = "POST", ...)
+    temp = postForm(myURL, style = "POST", .opts = curlOptions(useragent=R.version.string), ...)
   
   # The GET code that we used temporarily while NanoHTTPD POST was known to be busted.
   #
@@ -879,6 +891,11 @@ function(h2o, key) {
   .fetchJSON(h2o, key)
 }
 
+.check.exists <- function(h2o, key) {
+  keys <- as.data.frame(h2o.ls(h2o))[,1]
+  key %in% keys
+}
+
 #'
 #' Fetch the model from the key
 h2o.getModel <- function(h2o, key) {
@@ -916,14 +933,17 @@ h2o.getModel <- function(h2o, key) {
   dest_key   <- key #params$destination_key
 
   train_fr   <- new("H2OParsedData", key = "NA")
-  if(!is.null(response$"_dataKey")) train_fr <- h2o.getFrame(h2o, response$"_dataKey")
+  if(!is.null(response$"_dataKey") && .check.exists(h2o, response$"_dataKey")) {
+    train_fr <- h2o.getFrame(h2o, response$"_dataKey") } else {
+      train_fr@h2o <- h2o
+    }
   params$importance <- !is.null(params$varimp)
   if(!is.null(params$family) && model.type == "gbm_model") {
-    params$distribution <- "multinomial"
-    if(params$family == "AUTO") {
-      if(!is.null(json[[model.type]]$validAUC)) params$distribution <- "bernoulli"
+    if(params$classification == "false") {params$distribution <- "gaussian"
+      } else {
+        if(length(params$'_distribution') > 2) params$distribution <- "multinomial" else params$distribution <- "bernoulli"
+      }
     }
-  }
   if(algo == "model") {
     newModel <- new(model_obj, key = dest_key, data = train_fr, model = results_fun(json[[model.type]], train_fr, params))
     return(newModel)
