@@ -4,7 +4,7 @@
 # 3) If user does want to start H2O, but running non-locally, print an error
 h2o.init <- function(ip = "127.0.0.1", port = 54321, startH2O = TRUE, forceDL = FALSE, Xmx,
                      beta = FALSE, assertion = TRUE, license = NULL, nthreads = -2, max_mem_size = NULL, min_mem_size = NULL,
-                     ice_root = NULL, strict_version_check = TRUE) {
+                     ice_root = NULL, strict_version_check = TRUE, data_max_factor_levels = 1000000, many_cols = FALSE, chunk_bytes = 22) {
   if(!is.character(ip)) stop("ip must be of class character")
   if(!is.numeric(port)) stop("port must be of class numeric")
   if(!is.logical(startH2O)) stop("startH2O must be of class logical")
@@ -21,6 +21,18 @@ h2o.init <- function(ip = "127.0.0.1", port = 54321, startH2O = TRUE, forceDL =
   if(!is.null(license) && !is.character(license)) stop("license must be of class character")
   if(!is.null(ice_root) && !is.character(ice_root)) stop("ice_root must be of class character")
   if(!is.logical(strict_version_check)) stop("strict_version_check must be of class logical")
+  if(!is.numeric(data_max_factor_levels)) stop("`data_max_factor_levels` must be numeric.")
+  if(!is.logical(many_cols)) stop("`many_cols` must be logical")
+  if(!is.numeric(chunk_bytes)) stop("`chunk_bytes` must be numeric")
+
+  if ((R.Version()$major == "3") && (R.Version()$minor == "1.0")) {
+    warning("H2O is specifically not compatible with this exact")
+    warning("version of R 3.1.0.")
+    warning("Please change to a newer or older version of R.")
+    warning("(For technical details, search the r-devel mailing list")
+    warning("for type.convert changes in R 3.1.0.)")
+    stop("R 3.1.0 is not compatible with H2O!")
+  }
 
   if(!missing(Xmx)) {
     warning("Xmx is a deprecated parameter. Use `max_mem_size` and `min_mem_size` to set the memory boundaries. Using `Xmx` to set these.")
@@ -35,10 +47,10 @@ h2o.init <- function(ip = "127.0.0.1", port = 54321, startH2O = TRUE, forceDL =
   # Disable Up.json for now, since it doesn't exist in most builds.
   # Re-enable this sometime a few months from now.
   # myUpURL = paste("http://", ip, ":", port, "/Up.json", sep="")
-  myUpURL = paste("http://", ip, ":", port, sep="")
-  myURL = paste("http://", ip, ":", port, sep="")
+  myUpURL <- paste("http://", ip, ":", port, sep="")
+  myURL <- paste("http://", ip, ":", port, sep="")
   warnNthreads = FALSE
-  if(!url.exists(myUpURL)) {
+  if(!url.exists(myUpURL, .opts = curlOptions(useragent=R.version.string))) {
     if(!startH2O)
       stop(paste("Cannot connect to H2O server. Please check that H2O is running at", myURL))
     else if(ip == "localhost" || ip == "127.0.0.1") {
@@ -49,7 +61,12 @@ h2o.init <- function(ip = "127.0.0.1", port = 54321, startH2O = TRUE, forceDL =
         nthreads = 2
       }
 
-      .h2o.startJar(nthreads = nthreads, max_memory = max_mem_size, min_memory = min_mem_size, beta = beta, assertion = assertion, forceDL = forceDL, license = license, ice_root = ice_root)
+      .h2o.startJar(nthreads = nthreads, max_memory = max_mem_size,
+                    min_memory = min_mem_size, beta = beta,
+                    assertion = assertion, forceDL = forceDL,
+                    license = license, ice_root = ice_root,
+                    max_factor_levels = data_max_factor_levels,
+                    many_cols = many_cols, chunk_bytes = chunk_bytes)
 
       count = 0;
       while(!url.exists(myURL) && (count < 60)) {
@@ -129,7 +146,7 @@ h2o.clusterStatus <- function(client) {
   if(missing(client) || class(client) != "H2OClient") stop("client must be a H2OClient object")
   .h2o.__checkUp(client)
   myURL = paste("http://", client@ip, ":", client@port, "/", .h2o.__PAGE_CLOUD, sep = "")
-  res = fromJSON(postForm(myURL, .params = list(quiet="true", skip_ticks="true"), style = "POST"))
+  res = fromJSON(postForm(myURL, .params = list(quiet="true", skip_ticks="true"), style = "POST", .opts = curlOptions(useragent=R.version.string)))
   
   cat("Version:", res$version, "\n")
   cat("Cloud name:", res$cloud_name, "\n")
@@ -242,7 +259,11 @@ h2o.clusterStatus <- function(client) {
 #     h2o.shutdown(new("H2OClient", ip=ip, port=port), prompt = FALSE)
 # }
 
-.h2o.startJar <- function(nthreads = -1, max_memory = NULL, min_memory = NULL, beta = FALSE, assertion = TRUE, forceDL = FALSE, license = NULL, ice_root) {
+.h2o.startJar <- function(nthreads = -1, max_memory = NULL,
+                          min_memory = NULL, beta = FALSE,
+                          assertion = TRUE, forceDL = FALSE,
+                          license = NULL, ice_root, max_factor_levels = 1000000,
+                          many_cols = FALSE, chunk_bytes = 22) {
   command <- .h2o.checkJava()
 
   if (! is.null(license)) {
@@ -304,6 +325,9 @@ http://www.oracle.com/technetwork/java/javase/downloads/jdk7-downloads-1880260.h
   if(nthreads > 0) args <- c(args, "-nthreads", nthreads)
   if(beta) args <- c(args, "-beta")
   if(!is.null(license)) args <- c(args, "-license", license)
+  args <- c(args, "-data_max_factor_levels", max_factor_levels)
+  if(many_cols) args <- c(args, "-many_cols")
+  args <- c(args, "-chunk_bytes", chunk_bytes)
 
   cat("\n")
   cat(        "Note:  In case of errors look at the following log files:\n")
